/* internet.cpp reads TCP/IP addresses from a file and produces a 
 *  list of distinct addresses and a count of how many times each 
 *  appearedin the file.  The addresses and counts are stored in a 
 *  linked list.
 *
 * Input (keyboard): name of file containing addresses
 * Input (file):     addresses
 * Output:           a list of distinct addresses and their counts
 ****************************************************************/

#include <cassert>                  // assert
#include <string>                   // string
#include <iostream>                 // cin, cout, >>, <<
#include <iomanip>                  // setw()
#include <fstream>                  // ifstream, isopen()
#include <list>                     // list<T>
#include <algorithm>                // find
using namespace std;

//--------------- Begin class AddressItem -------------------------
class AddressCounter
{
 public:
   void read(istream & in) { in >> address; count = 1; }

   void print(ostream & out) const
   { out << setw(15) << left << address 
         << "\t occurs " << count << " times\n"; }

   void tally() { count++; }

   friend bool operator==(const AddressCounter& addr1,
                          const AddressCounter& addr2);



   string getAddress() const;

 private:
   string address;
   int count;
};


inline string AddressCounter::getAddress() const

{

	return address;

}


inline bool operator==(const AddressCounter& addr1,
                       const AddressCounter& addr2)
{ return addr1.getAddress() == addr2.getAddress(); }

//----------------- End class AddressCounter --------------------

typedef list<AddressCounter> TCP_IP_List;

int main()
{
   string fileName;                       // file of TCP/IP addresses
   TCP_IP_List addrCntList;               // list of addresses

   ifstream inStream;                     // open file of addresses
   cout << "Enter name of file containing TCP/IP addresses: ";
   cin >> fileName;
   inStream.open(fileName.data());
   assert(inStream.is_open());

   AddressCounter item;                   // one address & its count
   for (;;)                               // input loop:
   {
      item.read(inStream);                //   read an address
      if (inStream.eof()) break;          //   if eof, quit

      TCP_IP_List::iterator it =          //   is item in list?
        find(addrCntList.begin(), addrCntList.end(), item);
      if (it != addrCntList.end())        //   if so:
        (*it).tally();                    //    ++ its count
      else                                // otherwise
        addrCntList.push_back(item);      // add it to the list
   }                                      // end loop

   cout << "\nAddresses and Counts:\n\n";    // output the list
   for (TCP_IP_List::iterator it = addrCntList.begin();
                              it != addrCntList.end(); it++)
      (*it).print(cout);



   return 0;
}
